@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2009 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -52,7 +52,8 @@ gva_categories_init (GError **error)
         /* Convert Windows INI-style comments (lines beginning with ';')
          * to a form recognized by the GKeyFile parser. */
         cp = contents;
-        while (cp != NULL && *cp != '\0') {
+        while (cp != NULL && *cp != '\0')
+        {
                 if (*cp == ';')
                         *cp = '#';
                 /* Find the next line. */
