@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2009 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -20,15 +20,18 @@
 
 #include <errno.h>
 #include <string.h>
-#include <wait.h>
+#include <sys/wait.h>
 
 #ifdef HAVE_WORDEXP_H
 #include <wordexp.h>
 #endif
 
 #include "gva-error.h"
+#include "gva-input-file.h"
 #include "gva-mame-process.h"
+#include "gva-mute-button.h"
 #include "gva-preferences.h"
+#include "gva-ui.h"
 
 /*****************************************************************************
  * Private utilities for MAME backends
@@ -289,18 +292,17 @@ gva_mame_get_search_paths (const gchar *config_key,
  * gva_mame_get_input_files:
  * @error: return location for a @GError, or %NULL
  *
- * Returns a #GHashTable of input files for playback.  The keys of the
- * #GHashTable are absolute filenames and the values are the corresponding
- * game name.  If an error occurs, it returns %NULL and sets @error.
+ * Returns a list of #GvaInputFile instances corresponding to files in the
+ * input directory.  If an error occurs, it returns %NULL and sets @error.
  *
- * Returns: a #GHashTable of input files, or %NULL
+ * Returns: a list of #GvaInputFile instances, or %NULL
  **/
-GHashTable *
+GList *
 gva_mame_get_input_files (GError **error)
 {
-        GHashTable *hash_table;
         const gchar *basename;
         const gchar *directory;
+        GList *list = NULL;
         GDir *dir;
 
         directory = gva_mame_get_input_directory (error);
@@ -309,49 +311,20 @@ gva_mame_get_input_files (GError **error)
         if (dir == NULL)
                 return NULL;
 
-        hash_table = g_hash_table_new_full (
-                g_str_hash, g_str_equal,
-                (GDestroyNotify) g_free,
-                (GDestroyNotify) g_free);
-
         while ((basename = g_dir_read_name (dir)) != NULL)
         {
                 gchar *filename;
-                GIOChannel *channel;
-                GError *local_error = NULL;
+                GvaInputFile *input_file;
 
                 filename = g_build_filename (directory, basename, NULL);
-
-                channel = g_io_channel_new_file (filename, "r", &local_error);
-
-                if (channel != NULL)
-                {
-                        gchar name[16];
-                        GIOStatus status;
-
-                        status = g_io_channel_read_chars (
-                                channel, name, sizeof (name),
-                                NULL, &local_error);
-                        if (status == G_IO_STATUS_NORMAL)
-                                g_hash_table_insert (
-                                        hash_table, filename,
-                                        g_strdup (name));
-                        g_io_channel_unref (channel);
-                }
-
-                if (local_error != NULL)
-                {
-                        g_free (filename);
-                        g_hash_table_destroy (hash_table);
-                        g_propagate_error (error, local_error);
-                        hash_table = NULL;
-                        break;
-                }
+                input_file = gva_input_file_new (filename);
+                list = g_list_prepend (list, input_file);
+                g_free (filename);
         }
 
         g_dir_close (dir);
 
-        return hash_table;
+        return g_list_reverse (list);
 }
 
 /**
@@ -654,11 +627,22 @@ gva_mame_run_game (const gchar *name,
                         g_string_append (arguments, "-window ");
         }
 
-#ifdef WITH_WNCK
+#ifdef HAVE_WNCK
         if (gva_mame_supports_maximize ())
                 g_string_append (arguments, "-nomaximize ");
 #endif
 
+        if (gva_mame_supports_sound ())
+        {
+                GvaMuteButton *mute_button;
+
+                mute_button = GVA_MUTE_BUTTON (GVA_WIDGET_MAIN_MUTE_BUTTON);
+                if (gva_mute_button_get_muted (mute_button))
+                        g_string_append (arguments, "-nosound ");
+                else
+                        g_string_append (arguments, "-sound ");
+        }
+
         g_string_append_printf (arguments, "%s", name);
 
         /* Execute the command "${mame} ${name}". */
@@ -720,11 +704,22 @@ gva_mame_record_game (const gchar *name,
                         g_string_append (arguments, "-window ");
         }
 
-#ifdef WITH_WNCK
+#ifdef HAVE_WNCK
         if (gva_mame_supports_maximize ())
                 g_string_append (arguments, "-nomaximize ");
 #endif
 
+        if (gva_mame_supports_sound ())
+        {
+                GvaMuteButton *mute_button;
+
+                mute_button = GVA_MUTE_BUTTON (GVA_WIDGET_MAIN_MUTE_BUTTON);
+                if (gva_mute_button_get_muted (mute_button))
+                        g_string_append (arguments, "-nosound ");
+                else
+                        g_string_append (arguments, "-sound ");
+        }
+
         g_string_append_printf (arguments, "-record %s %s", inpname, name);
 
         /* Execute the command "${mame} -record ${inpname} ${name}". */
@@ -783,11 +778,22 @@ gva_mame_playback_game (const gchar *name,
                         g_string_append (arguments, "-window ");
         }
 
-#ifdef WITH_WNCK
+#ifdef HAVE_WNCK
         if (gva_mame_supports_maximize ())
                 g_string_append (arguments, "-nomaximize ");
 #endif
 
+        if (gva_mame_supports_sound ())
+        {
+                GvaMuteButton *mute_button;
+
+                mute_button = GVA_MUTE_BUTTON (GVA_WIDGET_MAIN_MUTE_BUTTON);
+                if (gva_mute_button_get_muted (mute_button))
+                        g_string_append (arguments, "-nosound ");
+                else
+                        g_string_append (arguments, "-sound ");
+        }
+
         g_string_append_printf (arguments, "%s -playback %s", name, inpname);
 
         /* Execute the command "${mame} ${name} -playback ${inpname}". */
