@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2009 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -55,7 +55,7 @@ gchar *         gva_mame_get_config_value       (const gchar *config_key,
 gboolean        gva_mame_has_config_value       (const gchar *config_key);
 gchar **        gva_mame_get_search_paths       (const gchar *config_key,
                                                  GError **error);
-GHashTable *    gva_mame_get_input_files        (GError **error);
+GList *         gva_mame_get_input_files        (GError **error);
 GvaProcess *    gva_mame_list_xml               (GError **error);
 gchar *         gva_mame_verify_roms            (const gchar *name,
                                                  GError **error);
