@@ -18,11 +18,9 @@
 
 #include "gva-ui.h"
 
-#include <glade/glade.h>
-#include <glade/glade-build.h>
-
 #include "gva-audit.h"
 #include "gva-column-manager.h"
+#include "gva-db.h"
 #include "gva-error.h"
 #include "gva-favorites.h"
 #include "gva-game-store.h"
@@ -36,7 +34,14 @@
 #include "gva-util.h"
 #include "gva-wnck.h"
 
-static GladeXML *xml = NULL;
+#ifdef HAVE_DBUS
+#include "gva-dbus.h"
+#endif
+
+#define SQL_INSERT_LASTPLAYED \
+        "INSERT INTO lastplayed VALUES ('%s', %" G_GINT64_FORMAT ")"
+
+static GtkBuilder *builder = NULL;
 static GtkUIManager *manager = NULL;
 static GtkActionGroup *action_group = NULL;
 static gboolean initialized = FALSE;
@@ -64,12 +69,54 @@ static const gchar *license =
 "along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 
 static void
-record_game_exited (GvaProcess *process, gint status, gchar *inpname)
+log_lastplayed (GvaProcess *process,
+                gint status,
+                gchar *name)
+{
+        GtkTreePath *path;
+        gchar *sql;
+        time_t now;
+        GError *error = NULL;
+
+        if (process->error != NULL)
+                return;
+
+        time (&now);
+
+        /* Record the time in the database. */
+        sql = g_strdup_printf (SQL_INSERT_LASTPLAYED, name, (gint64) now);
+        gva_db_execute (sql, &error);
+        gva_error_handle (&error);
+        g_free (sql);
+
+        /* Record the time in the tree model. */
+        path = gva_tree_view_lookup (name);
+        if (path != NULL)
+        {
+                GtkTreeModel *model;
+                GtkTreeIter iter;
+
+                model = gva_tree_view_get_model ();
+                gtk_tree_model_get_iter (model, &iter, path);
+
+                gtk_tree_store_set (
+                        GTK_TREE_STORE (model), &iter,
+                        GVA_GAME_STORE_COLUMN_LAST_PLAYED, &now, -1);
+
+                gtk_tree_path_free (path);
+        }
+
+        g_free (name);
+}
+
+static void
+record_game_exited (GvaProcess *process,
+                    gint status,
+                    gchar *inpname)
 {
         if (process->error == NULL)
                 gva_play_back_show (inpname);
 
-        g_object_unref (process);
         g_free (inpname);
 }
 
@@ -265,9 +312,25 @@ action_play_back_cb (GtkAction *action)
         process = gva_mame_playback_game (name, inpname, &error);
         gva_error_handle (&error);
 
+#ifdef HAVE_DBUS
+        if (process != NULL)
+        {
+                const gchar *reason;
+
+                /* Translators: This is passed through D-Bus as the
+                 * reason to inhibit GNOME screen saver. */
+                reason = _("Watching a fullscreen game");
+                gva_dbus_inhibit_screen_saver (process, reason, &error);
+                gva_error_handle (&error);
+        }
+#endif
+
         if (process != NULL)
         {
                 gva_wnck_listen_for_new_window (process, name);
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (log_lastplayed), g_strdup (name));
                 g_signal_connect_after (
                         process, "exited",
                         G_CALLBACK (g_object_unref), NULL);
@@ -379,13 +442,34 @@ action_record_cb (GtkAction *action)
         process = gva_mame_record_game (name, inpname, &error);
         gva_error_handle (&error);
 
+#ifdef HAVE_DBUS
+        if (process != NULL)
+        {
+                const gchar *reason;
+
+                /* Translators: This is passed through D-Bus as the
+                 * reason to inhibit GNOME screen saver. */
+                reason = _("Recording a fullscreen game");
+                gva_dbus_inhibit_screen_saver (process, reason, &error);
+                gva_error_handle (&error);
+        }
+#endif
+
         if (process != NULL)
         {
                 gva_wnck_listen_for_new_window (process, name);
                 g_signal_connect_after (
                         process, "exited",
-                        G_CALLBACK (record_game_exited), inpname);
+                        G_CALLBACK (log_lastplayed), g_strdup (name));
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (record_game_exited), g_strdup (inpname));
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (g_object_unref), NULL);
         }
+
+        g_free (inpname);
 }
 
 /**
@@ -533,9 +617,25 @@ action_start_cb (GtkAction *action)
         process = gva_mame_run_game (name, &error);
         gva_error_handle (&error);
 
+#ifdef HAVE_DBUS
+        if (process != NULL)
+        {
+                const gchar *reason;
+
+                /* Translators: This is passed through D-Bus as the
+                 * reason to inhibit GNOME screen saver. */
+                reason = _("Playing a fullscreen game");
+                gva_dbus_inhibit_screen_saver (process, reason, &error);
+                gva_error_handle (&error);
+        }
+#endif
+
         if (process != NULL)
         {
                 gva_wnck_listen_for_new_window (process, name);
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (log_lastplayed), g_strdup (name));
                 g_signal_connect_after (
                         process, "exited",
                         G_CALLBACK (g_object_unref), NULL);
@@ -825,27 +925,17 @@ ui_init (void)
                 G_N_ELEMENTS (view_radio_entries),
                 0, G_CALLBACK (action_view_changed_cb), NULL);
 
-#define REGISTER_WIDGET(type) \
-        (glade_register_widget \
-        ((type), glade_standard_build_widget, NULL, NULL))
-
-        filename = gva_find_data_file (PACKAGE ".glade");
-        if (filename != NULL)
+        filename = gva_find_data_file (PACKAGE ".builder");
+        if (filename != NULL && error == NULL)
         {
-                REGISTER_WIDGET (GTK_TYPE_LINK_BUTTON);
-                REGISTER_WIDGET (GVA_TYPE_COLUMN_MANAGER);
-                REGISTER_WIDGET (GVA_TYPE_MUTE_BUTTON);
-
-                glade_provide ("gva");
-                xml = glade_xml_new (filename, NULL, NULL);
-                glade_xml_signal_autoconnect (xml);
+                builder = gtk_builder_new ();
+                gtk_builder_add_from_file (builder, filename, &error);
+                gtk_builder_connect_signals (builder, NULL);
         }
         g_free (filename);
 
-#undef REGISTER_WIDGET
-
         filename = gva_find_data_file (PACKAGE ".ui");
-        if (filename != NULL)
+        if (filename != NULL && error == NULL)
         {
                 manager = gtk_ui_manager_new ();
                 gtk_ui_manager_insert_action_group (manager, action_group, 0);
@@ -865,7 +955,7 @@ ui_init (void)
         if (error != NULL)
                 g_error ("%s", error->message);
 
-        if (xml == NULL || manager == NULL)
+        if (builder == NULL || manager == NULL)
                 g_error ("%s", _("Failed to initialize user interface"));
 }
 
@@ -909,16 +999,16 @@ gva_ui_get_action (const gchar *action_name)
 GtkWidget *
 gva_ui_get_widget (const gchar *widget_name)
 {
-        GtkWidget *widget;
+        GObject *object;
 
         g_return_val_if_fail (widget_name != NULL, NULL);
 
         if (G_UNLIKELY (!initialized))
                 ui_init ();
 
-        widget = glade_xml_get_widget (xml, widget_name);
-        g_assert (widget != NULL);
-        return widget;
+        object = gtk_builder_get_object (builder, widget_name);
+        g_assert (object != NULL);
+        return GTK_WIDGET (object);
 }
 
 /**
