@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2009 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -39,16 +39,22 @@ G_BEGIN_DECLS
  *      Print shell commands invoking MAME.
  * @GVA_DEBUG_SQL:
  *      Print SQL commands to the game database.
+ * @GVA_DEBUG_IO:
+ *      Print all communication between GVA and MAME.
+ * @GVA_DEBUG_INP:
+ *      Print information about input files.
  *
  * These flags indicate which types of debugging messages will be triggered
  * at runtime. Debugging messages can be triggered by setting the GVA_DEBUG
- * environment variable to a colon-separated list of "mame" and "sql".
+ * environment variable to a colon-separated list of "mame", "sql" and "io".
  **/
 typedef enum
 {
-        GVA_DEBUG_NONE  = 0,
-        GVA_DEBUG_MAME  = 1 << 0,
-        GVA_DEBUG_SQL   = 1 << 1
+        GVA_DEBUG_NONE = 0,
+        GVA_DEBUG_MAME = 1 << 0,
+        GVA_DEBUG_SQL  = 1 << 1,
+        GVA_DEBUG_IO   = 1 << 2,
+        GVA_DEBUG_INP  = 1 << 3
 } GvaDebugFlags;
 
 gchar *         gva_choose_inpname              (const gchar *game);
